﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using FLogger4net.Constant;
using FLogger4net.strategy;
using FLogger4net.utils;
using System.Threading;

namespace FLogger4net
{
    //http://www.cnblogs.com/cyfonly/p/6139049.html
    public class FLogger
    {
        private  static readonly Object obejct = new object();
        private static FLogger instance;

        private static LogManager logManager = LogManager.Instance;

        public static FLogger getInstance()
        {
            if (instance == null)
            {
                lock(obejct)
                {
                    if (instance == null)
                    {
                        instance = new FLogger();
                    }
                }
            }
            return instance;
        }
        public FLogger() 
        {
            Thread thread = new Thread(logManager.run);
            thread.Start();
        }
        
        //调试日志
        public void debug(String logMsg)
        {
            writeLog("debug", ConstantCLS.DEBUG, logMsg);
        }

        //写普通日志
        public void info(String logMsg)
        {
            writeLog("info", ConstantCLS.INFO, logMsg);
        }

        //写警告日志
        public void warn(String logMsg)
        {
            writeLog("warn", ConstantCLS.WARN, logMsg);
        }

        //写错误日志
        public void error(String logMsg)
        {
            writeLog("error", ConstantCLS.ERROR, logMsg);
        }

        //写严重错误日志
        public void fatal(String logMsg)
        {
            writeLog("fatal", ConstantCLS.FATAL, logMsg);
        }

        //写系统日志
        public void writeLog(int level, String logMsg)
        {
            // writeLog(Constant.LOG_DESC_MAP.get(level.ToString().toLowerCase(), level, logMsg);
        }

        //写日志
        public void writeLog(String logFileName, int level, String logMsg)
        {
            if (logMsg != null && ConstantCLS.CFG_LOG_LEVEL.IndexOf("" + level) >= 0)
            {
                StringBuilder sb = new StringBuilder(logMsg.Length + 100);
                sb.Append("[");
                sb.Append(ConstantCLS.LOG_DESC_MAP[level.ToString()]);
                sb.Append("] ");
                sb.Append(TimeUtil.getFullDateTime());
                sb.Append(" [");
                sb.Append(Thread.CurrentThread.Name);
                sb.Append("] ");
                sb.Append(logMsg);
                sb.Append("\n");
                logManager.addLog(logFileName, sb);

                //错误信息同时打印到控制台
                if (ConstantCLS.ERROR == level || ConstantCLS.FATAL == level)
                {
                    try
                    {
                        Console.WriteLine(CommUtil.GetBytesOfString(ConstantCLS.CFG_CHARSET_NAME,sb.ToString()) + ConstantCLS.CFG_CHARSET_NAME);
                    }
                    catch (Exception e)
                    {
                        Console.WriteLine(CommUtil.getExpStack(e));
                    }
                }
            }
        }
        
        //关闭
        private void close()
        {
            logManager.close();
        }
    }
}
