﻿using FLogger4net.Constant;
using FLogger4net.strategy;
using FLogger4net.utils;
using System;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace FLogger4net.strategy
{
    public class LogManager
    {

        private static volatile LogManager instance = null;

        private static string strInstanceName = "";


        /** 日志文件列表 */
        private ConcurrentDictionary<string, LogFileItem> logFileMap = new ConcurrentDictionary<string, LogFileItem>();

        /** 日志写入的间隔时间 */
        public static long WRITE_LOG_INV_TIME = CommUtil.getConfigByLong("WRITE_LOG_INV_TIME", 1000);

        /** 单个日志文件的大小(默认为10M) */
        public static long SINGLE_LOG_FILE_SIZE = CommUtil.getConfigByLong("SINGLE_LOG_FILE_SIZE", 1024 * 1024);

        /** 缓存大小(默认10KB) */
        public static long SINGLE_LOG_CACHE_SIZE = CommUtil.getConfigByLong("SINGLE_LOG_CACHE_SIZE", 1024 * 10);

        private static long Jan1st1970Ms = new System.DateTime(1970, 1, 1, 0, 0, 0, System.DateTimeKind.Utc).Ticks;
        /** 是否运行 */
        private bool bIsRun = true;

        public LogManager() { }
        private static object syncRoot = new Object();


        public static LogManager Instance
        {
            get
            {
                if (instance == null)
                {
                    lock (syncRoot)
                    {
                        if (instance == null)
                            instance = new LogManager();
                    }
                }
                return instance;
            }
        }

        //添加日志
        public void addLog(String logFileName, StringBuilder logMsg)
        {
            //获得单个日志文件的信息
            LogFileItem lfi = null;
            if (!logFileMap.ContainsKey(logFileName))
            {
                lock (this)
                {
                    if (!logFileMap.ContainsKey(logFileName))
                    {
                        lfi = new LogFileItem();
                        lfi.logFileName = logFileName;
                        lfi.nextWriteTime = (System.DateTime.UtcNow.Ticks - Jan1st1970Ms) / 10000 + WRITE_LOG_INV_TIME;
                        logFileMap.TryAdd(logFileName, lfi);
                    }
                    else
                    {
                        lfi = logFileMap[logFileName];
                    }
                }
            }
            else
            {
                lfi = logFileMap[logFileName];
            }
            //同步单个文件的日志
            lock (lfi)
            {
                if (lfi.currLogBuff == 'A')
                {
                    lfi.alLogBufA.Add(logMsg);
                }
                else
                {
                    lfi.alLogBufB.Add(logMsg);
                }
                lfi.currCacheSize += System.Text.Encoding.UTF8.GetBytes(logMsg.ToString()).Length;
            }
        }
        //创建日志文件
        private void createLogFile(LogFileItem lfi)
        {
            //当前系统日期
            String currPCDate = TimeUtil.getPCDate('-');

            //判断日志root路径是否存在，不存在则先创建
            if (Directory.Exists(ConstantCLS.CFG_LOG_PATH))
            {
                if (Directory.Exists(ConstantCLS.CFG_LOG_PATH))
                {
                    Directory.CreateDirectory(ConstantCLS.CFG_LOG_PATH);
                }
            }

            //如果超过单个文件大小，则拆分文件
            if (lfi.fullLogFileName != null && lfi.fullLogFileName.Length > 0 && lfi.currLogSize >= LogManager.SINGLE_LOG_FILE_SIZE)
            {
                if (File.Exists(lfi.fullLogFileName))
                {
                    String newFileName = ConstantCLS.CFG_LOG_PATH + "/" + lfi.lastPCDate + "/" + lfi.logFileName + "_" + TimeUtil.getPCDate() + "_" + TimeUtil.getCurrTime() + ".log";

                    try
                    {
                        File.Move(lfi.fullLogFileName, newFileName);
                        MessageBox("日志已自动备份为 " + newFileName + "成功!");
                        lfi.fullLogFileName = "";
                        lfi.currLogSize = 0;
                    }
                    catch (Exception ex)
                    {
                        MessageBox("日志已自动备份为 " + newFileName + "失败!"+ex.ToString());
                    }
                }
            }
            //创建文件
            if (lfi.fullLogFileName == null || lfi.fullLogFileName.Length <= 0 || !lfi.lastPCDate.Equals(currPCDate))
            {
                String sDir = ConstantCLS.CFG_LOG_PATH + "/" + currPCDate;
                if (!Directory.Exists(sDir))
                {
                    DirectoryInfo dirInfo = Directory.CreateDirectory(sDir);
                }
                lfi.fullLogFileName = sDir + "/" + lfi.logFileName + ".log";
                lfi.lastPCDate = currPCDate;

                if (File.Exists(lfi.fullLogFileName))
                {
                    FileInfo fi = new FileInfo(lfi.fullLogFileName);
                    lfi.currLogSize = fi.Length;
                }
                else
                {
                    File.Create(lfi.fullLogFileName);
                    lfi.currLogSize = 0;
                }
            }
        }

        private void MessageBox(object p)
        {
            //throw new NotImplementedException();
        }
        /// <summary>
        /// 输出日志到文件
        /// </summary>
        /// <param name="sFullFileName">完整的日志文件名称</param>
        /// <param name="sbLogMsg">日志文件内容</param>
        /// <returns>返回输出内容大小</returns>
        private int writeToFile(String sFullFileName, List<StringBuilder> sbLogMsg)
        {
            int size = 0;
            //FileStream fs = new FileStream(sFullFileName, FileMode.Open, FileAccess.ReadWrite); //可以指定盘符，也可以指定任意文件名，还可以为word等文件
            StreamWriter sw = new StreamWriter(sFullFileName,true); // 创建写入流

            try
            {
                for (int i = 0; i < sbLogMsg.Count; i++)
                {
                    StringBuilder logMsg = sbLogMsg[i];
                    byte[] tmpBytes = CommUtil.StringToBytes(logMsg.ToString());

                    sw.WriteLine(logMsg.ToString()); // 写入Hello World
                    size += logMsg.Length;//TODO需要判断写二进制和字符串的大小

                }
                sw.Close(); //关闭文件
                //fs.Close();
                sbLogMsg.Clear();
            }
            catch (Exception ex)
            {
                throw new Exception("写日志出错" + ex.ToString());
            }
            finally
            {
                if (sw != null)
                {
                    sw.Close();
                }
                //if (fs != null)
                //{
                //    fs.Close();
                //}
            }
            return size;
        }
        //关闭方法
        public void close()
        {
            bIsRun = false;
            try
            {
                flush(true);
            }
            catch (Exception ex)
            {
                System.Console.WriteLine("关闭日志服务错误..." + ex.ToString());
                Console.WriteLine(System.Environment.StackTrace);
            }
        }

        // 线程方法
        public void run()
        {
            int i = 0;
            while (bIsRun)
            {
                try
                {
                    //输出到文件
                    flush(false);
                    //重新获取日志级别
                    if (i++ % 100 == 0)
                    {
                        ConstantCLS.CFG_LOG_LEVEL = CommUtil.getConfigByString("LOG_LEVEL", "0,1,2,3,4");
                        i = 1;
                    }
                }
                catch (Exception ex)
                {
                    MessageBox("开启日志服务错误..." + ex.ToString());
                }
            }
        }

        /// <summary>
        /// 输出缓存的日志到文件
        /// </summary>
        /// <param name="bIsForce">是否强制将缓存中的日志输出到文件</param>
        private void flush(bool bIsForce)
        {
            long currTime = new System.DateTime(1970, 1, 1, 0, 0, 0, System.DateTimeKind.Utc).Ticks;

            foreach ( var item in logFileMap)
            {
                LogFileItem lfi = item.Value;
                if (/*currTime >= lfi.nextWriteTime ||*/ SINGLE_LOG_CACHE_SIZE <= lfi.currCacheSize || bIsForce == true)
                {
                    //获得需要进行输出的缓存列表
                    List<StringBuilder> alWrtLog = null;
                    lock (lfi)
                    {
                        if (lfi.currLogBuff == 'A')
                        {
                            alWrtLog = lfi.alLogBufA;
                            lfi.currLogBuff = 'B';
                        }
                        else
                        {
                            alWrtLog = lfi.alLogBufB;
                            lfi.currLogBuff = 'A';
                        }
                        lfi.currCacheSize = 0;
                    }
                    //创建日志文件
                    createLogFile(lfi);
                    //输出日志
                    int iWriteSize = writeToFile(lfi.fullLogFileName, alWrtLog);
                    lfi.currLogSize += iWriteSize;
                }
            }
        }

    }
}
